#ontop {
    position: fixed;
    display: none;
    width: 100%;
    height: 100%;
    top:0;
    bottom: 0;
    left:0;
    right: 0;
    background-color: rgba(0,0,0,0.5);
    z-index: 1001; /* 1001 coz sidebar is 1000. So will be on top of sidebar*/
}

#state {
    position: absolute;
    top: 50%;
    left: 50%;
    font-size: 30px;
    color: white;
}
