{% extends "base.html" %}
<!DOCTYPE html>
{% block pagetitle %} Change Password {% endblock %}

{% block content %}
<form action="" method="post" >
  {% csrf_token %}
  <center>
  <table class=span1>
  {{ form }}
  </table>
  </center>
  <center><button class="btn" type="submit">Change Password</button>
  <button class="btn" type="button" name="button" onClick='location.replace("{{URL_ROOT}}/exam/");'>Cancel</button></center>
</form>
{% endblock content %}
