{% extends "base.html" %}
{% block pagetitle %}Reset Password{% endblock %}

{% block content  %}
    {% if validlink %}
        <p>Please enter your new password twice so we can verify you typed it in correctly</p>.
        <form method="post">
            {% csrf_token %}
            {{ form.as_p }}
            <button class= "btn" type="submit">Submit</button>
        </form>
    {% else %}
        <p>This reset link is no longer valid!</p>
    {% endif %}
{% endblock %}
