# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2017-03-31 06:32
from __future__ import unicode_literals

import datetime
from django.db import migrations, models
import django.utils.timezone
from django.utils.timezone import utc


class Migration(migrations.Migration):

    dependencies = [
        ('yaksh', '0003_auto_20170321_0917'),
    ]

    operations = [
        migrations.AddField(
            model_name='course',
            name='end_enroll_time',
            field=models.DateTimeField(default=datetime.datetime(2199, 1, 1, 0, 0, tzinfo=utc), null=True, verbose_name='End Date and Time for enrollment of course'),
        ),
        migrations.AddField(
            model_name='course',
            name='start_enroll_time',
            field=models.DateTimeField(default=django.utils.timezone.now, null=True, verbose_name='Start Date and Time for enrollment of course'),
        ),
    ]
