{% extends "base.html" %}
{% block title %}Password reset successful{% endblock %}
{% block pagetitle %} Instruction for setting new password has been mailed to your registered email address {% endblock %}
