# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2017-04-10 10:24
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('yaksh', '0004_auto_20170331_0632'),
    ]

    operations = [
        migrations.AddField(
            model_name='assignmentupload',
            name='question_paper',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='yaksh.QuestionPaper'),
        ),
        migrations.AlterField(
            model_name='hooktestcase',
            name='hook_code',
            field=models.TextField(default='def check_answer(user_answer):\n   \'\'\' Evaluates user answer to return -\n   success - Boolean, indicating if code was executed correctly\n   mark_fraction - Float, indicating fraction of the\n                  weight to a test case\n   error - String, error message if success is false\n\n   In case of assignment upload there will be no user answer \'\'\'\n\n   success = False\n   err = "Incorrect Answer" # Please make this more specific\n   mark_fraction = 0.0\n\n   # write your code here\n\n   return success, err, mark_fraction\n\n'),
        ),
    ]
