# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2017-11-14 14:29
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('yaksh', '0007_alter_profile_activation_key'),
    ]

    operations = [
        migrations.AddField(
            model_name='answerpaper',
            name='questions_order',
            field=models.TextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='question',
            name='snippet',
            field=models.TextField(blank=True),
        ),
    ]
