{% extends "manage.html" %}


{% block subtitle %}Questions in "{{ papers.quiz.description }}"{% endblock %}

{% block script %}
<script src="{{ URL_ROOT }}/static/yaksh/js/show_question.js"></script>
{% endblock %}

{% block manage %}
<form name=frm action="" method="post">
{% csrf_token %}
<p><u> Fixed Questions </u></p>
{% for q in fixed_questions %}
<input type="checkbox" name="papers" value="{{ q.id }}">&nbsp;&nbsp;<a href="{{URL_ROOT}}/exam/manage/addquestion/{{ q.id }}">{{ q.summary}}</a><br>
{% endfor %}
<p><u> Random Questions </u></p>
{% for random_set in random_questions %}
<p>{{ random_set.num_questions}} question(s) from {{ random_set.questions.all|length }} question(s)</p>
{% for q in random_set.questions.all %}
<input type="checkbox" name="papers" value="{{ q.id }}">&nbsp;&nbsp;<a href="{{URL_ROOT}}/exam/manage/addquestion/{{ q.id }}">{{ q.summary}}</a><br>
{% endfor %}
{% endfor %}
<br>
</form>
{% endblock %}
