{% extends "base.html" %}
{% block pagetitle %}Password reset complete{% endblock %}
{% block content %}
<p>Your password has been reset. </p>
<p><a href="{{ login_url }}">Log in</a></p>
{% endblock %}
