# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2017-03-21 09:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('yaksh', '0002_questionpaper_fixed_question_order'),
    ]

    operations = [
        migrations.CreateModel(
            name='FloatTestCase',
            fields=[
                ('testcase_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='yaksh.TestCase')),
                ('correct', models.FloatField(default=None)),
                ('error_margin', models.FloatField(blank=True, default=0.0, help_text='Margin of error', null=True)),
            ],
            bases=('yaksh.testcase',),
        ),
        migrations.CreateModel(
            name='IntegerTestCase',
            fields=[
                ('testcase_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='yaksh.TestCase')),
                ('correct', models.IntegerField(default=None)),
            ],
            bases=('yaksh.testcase',),
        ),
        migrations.CreateModel(
            name='StringTestCase',
            fields=[
                ('testcase_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='yaksh.TestCase')),
                ('correct', models.TextField(default=None)),
                ('string_check', models.CharField(choices=[('lower', 'Case Insensitive'), ('exact', 'Case Sensitive')], max_length=200)),
            ],
            bases=('yaksh.testcase',),
        ),
        migrations.AlterField(
            model_name='question',
            name='type',
            field=models.CharField(choices=[('mcq', 'Single Correct Choice'), ('mcc', 'Multiple Correct Choices'), ('code', 'Code'), ('upload', 'Assignment Upload'), ('integer', 'Answer in Integer'), ('string', 'Answer in String'), ('float', 'Answer in Float')], max_length=24),
        ),
        migrations.AlterField(
            model_name='testcase',
            name='type',
            field=models.CharField(choices=[('standardtestcase', 'Standard Testcase'), ('stdiobasedtestcase', 'StdIO Based Testcase'), ('mcqtestcase', 'MCQ Testcase'), ('hooktestcase', 'Hook Testcase'), ('integertestcase', 'Integer Testcase'), ('stringtestcase', 'String Testcase'), ('floattestcase', 'Float Testcase')], max_length=24, null=True),
        ),
    ]
