{% extends "base.html" %}
{% block nav %}
<nav class="navbar navbar-fixed-top navbar-inverse">
      <div class="container">
        <div class="navbar-header">
              <button type="button" class="navbar-toggle" data-toggle="collapse" data-target="#navbar">
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
                <span class="icon-bar"></span>
              </button>
          <a class="navbar-brand" href="{{ URL_ROOT }}/exam/manage/"> Online Test </a>
        </div>
        <div id="navbar" class="collapse navbar-collapse">
          <ul class="nav navbar-nav">
            <li><a href="{{ URL_ROOT }}/exam/manage/questions">Questions</a></li>
            <li><a href="{{ URL_ROOT }}/exam/manage/gradeuser">Grade User</a></li>
            <li><a href="{{ URL_ROOT }}/exam/manage/monitor">Monitor</a></li>
            <li><a href="{{ URL_ROOT }}/exam/manage/courses">Courses</a></li>
            <li><a href="{{ URL_ROOT }}/exam/viewprofile">My Profile</a></li>
            <li><a href="{{ URL_ROOT }}/exam/reset/changepassword">Change Password</a></li>
            <li><a href="{{ URL_ROOT }}/exam/manage/grader"> Grader </a></li>
            <li><a style='cursor:pointer' onClick='location.replace("{{URL_ROOT}}/exam/logout/");'>Log out</a></li>
          </ul>
        </div><!-- /.navbar -->
      </div><!-- /.container -->
    </nav><!-- /.navbar -->
{% endblock %}
{% block content %}
    {% block main %}
    {% endblock %}
{% endblock %}
