{% extends "base.html" %}
{% block pagetitle %} Email will be send to the registered email address {% endblock %}
{% block content %}
<form action="" method="post">
    {% csrf_token %}
    {{ form }}
    <button class="btn" type="submit">Request</button>
    <a class="btn" href="{{URL_ROOT}}/exam/login/">Cancel</a>
</form>
{% endblock content %}
