{% extends "base.html" %}

{% block pagetitle %}YAKSH {% endblock %}

{% block content %}
    <div class="row">
        <div class="col-md-6" style="border-right:1px solid gray;">
        <form action="" method="post">
        {% csrf_token %}
          
        <center><table class="table">
        {{ form.as_table }}
        </table></center>
        <button class="btn btn-primary" type="submit">Login</button>
        <a class="btn btn-primary" href="{{URL_ROOT}}/exam/reset/forgotpassword/">Forgot Password?</a>
        <a class="btn btn-primary" href="{{URL_ROOT}}/exam/register/">New User? Sign-Up </a>
        <h3> Login with </h3>
        <a class="btn btn-social-icon btn-google" href="{% url 'social:begin' 'google-oauth2' %}">
            <span class="fa fa-google" style="font-size:32px;color:red" ></span>
        </a>
        <a class="btn btn-social-icon btn-facebook"  href="{% url 'social:begin' 'facebook' %}">
            <span class="fa fa-facebook-square" style="font-size:32px"></span>
        </a>
        </form>
        </div>
        <div class="col-md-6">
            <p> Online Test Interface for conducting online programming quiz.
            <ul>
                <li>Define fairly complicated programming problems and have users
                solve the problem. </li>
                <li>Immediate verification of code solution. </li>
                <li>Supports pretty much arbitrary programming questions in Python, C, C++ and
                simple Bash and uses "test cases" to test the implementations of the students.</li>
                <li>Supports simple multiple choice questions and File uploads.</li>
                <li>Since it runs on your Python, you could technically test any Python based library.</li>
                <li>Scales to over 500+ simultaneous users.</li>
            </ul>
            </p>
            <br/>
            <p><b>Fork us at:</b>
            <a class = "btn btn-social-icon btn-github"
            href ="https://github.com/fossee/online_test">
            
            <span class="fa fa-github" style="font-size:48px"></span>
            </p>
            </a>
        </div>
    </div>
      
{% endblock content %}
