{% extends "manage.html" %}

{% block subtitle %}Add Course{% endblock %}

{% block css %}
<link rel="stylesheet" href="{{ URL_ROOT }}/static/yaksh/css/question_quiz.css" type="text/css" />
<link rel="stylesheet" href="{{ URL_ROOT }}/static/yaksh/css/jquery.datetimepicker.css" type="text/css" /> 
{% endblock %}

{% block script %}
<script src="{{ URL_ROOT }}/static/yaksh/js/jquery.datetimepicker.full.min.js"></script>
{% endblock %}

{% block content %}
<form name=frm id=frm action="" method="post" >
  {% csrf_token %}
  <center>
  <table class=span1>
  {{ form.as_table }}
  </table>
  <br/>
  <script type="text/javascript">
    $("#id_start_enroll_time").datetimepicker({format: 'Y-m-d H:i:s'});
    $("#id_end_enroll_time").datetimepicker({format: 'Y-m-d H:i:s'});
  </script>
  </center>

  <center><button class="btn btn-default" type="submit" id="submit" name="questionpaper">Save </button>
  <button class="btn btn-default" type="button" name="button" onClick='location.replace("{{URL_ROOT}}/exam/manage/");'>Cancel</button> </center>
</form>
{% endblock %}
