{% extends "manage.html" %}


{% block subtitle %} 
List of Users 
{% endblock %}

{% block css %}
<link rel="stylesheet" href="{{ URL_ROOT }}/static/yaksh/css/showusers.css" type="text/css" /> 
{% endblock css %}


{% block manage %}
<center><table style="text-align:left;" class=table-class border=3>
<th>Username
<th>First Name
<th>Last Name
<th>Quiz Description
{% for papers in question %}
<tr><td><a href="{{URL_ROOT}}/exam/manage/gradeuser/{{ papers.user.username }}">{{ papers.user.username }}</a><br>
    <td >{{ papers.user.first_name.title }}
    <td>{{ papers.user.last_name.title }}
    <td>{{ papers.question_paper.quiz.description }}
{% endfor %}
</table></center>
{% endblock %}
