# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2017-03-20 13:32
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('yaksh', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='questionpaper',
            name='fixed_question_order',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]
