{% extends "user.html" %}

{% block pagetitle %} Results {% endblock pagetitle %}
{% block content %}
<form>
  {% csrf_token %}
  <center><table class="table well">
    <th>Quiz Description
    <th>Obtained Marks
    <th>Maximum Marks
    <th>Percentage
    {% for paper in papers %}
        <tr>
            <td>{{ paper.question_paper.quiz.description }}</td>
            <td>{{ paper.marks_obtained }}</td>
            <td>{{ paper.question_paper.total_marks }}</td>
            <td>{{ paper.percent }}</td>
        </tr>
    {% endfor %}
  </table></center>
</form>
{% endblock %}

