body {
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
  font-size: 13px;
  font-weight: normal;
  line-height: 18px;
  color: #404040;
}
.tabs li {
    text-align: center;
    width: 33%;
}
.tabs li:last-child {
    width: 34%;
}
.tabs > .active > a {
    border: 0;
    background: lightgreen;
}
.tabs > .active > a:hover {
    border: 0;
    background: green;
    color: #ffffff;
}
.tabs li a {
    border-radius: 0;
    margin-right: 0;
}
.tabs {
    border: 1px solid #ddd;
}
#progress {
    background: red;
}
#content-left{
    text-align: center;
    background: grey;
}
#content-right{
    text-align: center;
    background: grey;
}
#selectors {
    margin-left: 0;
    background: #fafafa;
    padding: 7px 0;
    border: 2px solid #f5f5f5;
}
#selectors .col-md-6 {
    margin-left: 0;
}
#id_question_type {
    width: 100%;
}
#id_marks {
    width: 100%;
}
#fixed-questions .col-md-6 > div,
#random-questions .col-md-6 > div{
    background: #f5f5f5;
    height: 200px;
    border: 1px solid #333333;
    padding: 5px;
}
#fixed-available,
#random-available {
    height: 125px;
    min-height: 125px;
    overflow-y: scroll;
    margin-bottom: 15px;
}
#fixed-added,
#random-added {
    height: 160px;
    overflow-y: scroll;
}
#fixed-added hr,
#random-added hr {
    margin: 5px 0 4px;
}
.qcard {
    position: relative;
    background: #ffffff;
    padding: 5px;
    margin: 5px 5px;
    box-shadow: 1px 1px 5px #cccccc;
    -webkit-box-shadow: 1px 1px 5px #cccccc;
    -moz-box-shadow: 1px 1px 5px #cccccc;
    -o-box-shadow: 1px 1px 5px #cccccc;
}
.qcard ul {
    margin-bottom: 5px;
}
.qcard .remove {
    position: absolute;

    top: 3px;
    right: 3px;
    padding: 1px 3px;
    text-decoration: none;
    color: #ffffff;
    background: #ff4136;
    border-radius: 3px;
    font-weight: bold;
}
.qcard .remove:hover {
    background: #333333;
}
.red-alert {
    border: 2px solid red;
}
.well{
    padding: 5px;
}
