{% extends "base.html" %}

{% block pagetitle %}Password change successful{% endblock %}
{% block script %}
<script type="text/javascript">
	window.setTimeout(function()
	{   
		location.href="{{ URL_ROOT }}/exam/"
	}, 2000);
</script>
{% endblock %}
{% block content %}
<h3>Your password has been changed successfully.</h3>
<h4>Redirecting ...</h4>
{% endblock %}
